import { errorLog } from 'ic-logs';

import CODE_MESSAGES from './constants/codeMessages';
import { COMPARABLE_TYPES_AND_OPERATORS } from './constants/comparableAndOperators';
import Exceptions from './exceptions/exceptions';
import IComparableAndOperator from './interfaces/IComparableAndOperator';
import ICompare from './interfaces/ICompare';
import IOperator from './interfaces/IOperators';
import { isSameWeek } from './utils/date';
import DateComparatorHelper from './utils/date_comparator';
import DateString from './utils/date_string';

export default class OperatorValidator {
  type_of_value: ICompare;

  value: unknown;

  value_to_compare: unknown;

  comparable_types_and_operators: IComparableAndOperator

  operator: IOperator;

  constructor(type_of_value: ICompare, value: unknown, value_to_compare: unknown, operator?: IOperator) {
    this.type_of_value = type_of_value;
    this.value = value;
    this.value_to_compare = value_to_compare;
    this.operator = operator || 'EQUALS';
    this.comparable_types_and_operators = COMPARABLE_TYPES_AND_OPERATORS;
  }

  public compare(): boolean {
    try {
      this.verifyTypeAndOperator();
      const response = this.compareOperators();
      return response;
    } catch (error) {
      errorLog('[Compare]:', error.message);
      return false;
    }
  }

  private setDateType(value_date: DateString): void {
    if (this.comparable_types_and_operators.string.includes(this.operator)) return;

    const date_comparator_helper = new DateComparatorHelper(this.operator, value_date.value, this.value_to_compare);

    this.type_of_value = date_comparator_helper.type_of;
    this.value = date_comparator_helper.primary_value_to_compare;
    this.value_to_compare = date_comparator_helper.secondary_value_to_compare;
  }

  private verifyTypeAndOperator() {
    if (this.type_of_value === 'string') {
      const date_string = new DateString(this.value as string);
      if (date_string.is_date) this.setDateType(date_string);
    }

    const operators = this.comparable_types_and_operators[this.type_of_value];

    if (!operators || !operators.includes(this.operator)) {
      errorLog(`[INVALID_OPERATOR]: {"operator": "${this.operator}", "type": "${this.type_of_value}", "value": "${this.value}}`);
      throw new Exceptions.BusinessError(CODE_MESSAGES.INVALID_OPERATOR);
    }
  }

  private compareOperators(): boolean {
    let array: Array<typeof this.value>;
    let value_date: Date;
    let value_to_compare_date: Date;

    switch (this.operator) {
      case 'BIGGER':
        return this.value > this.value_to_compare;

      case 'BIGGEREQ':
        return this.value >= this.value_to_compare;

      case 'DIFF':
        return this.value !== this.value_to_compare;

      case 'ENDING':
        return `${this.value}`.endsWith(`${this.value_to_compare}`);

      case 'MINOR':
        return this.value < this.value_to_compare;

      case 'MINOREQ':
        return this.value <= this.value_to_compare;

      case 'STARTING':
        return `${this.value}`.startsWith(`${this.value_to_compare}`);

      case 'IN':
        array = this.value_to_compare as Array<typeof this.value_to_compare>;
        return array.includes(this.value);

      case 'NOT_IN':
        array = this.value_to_compare as Array<typeof this.value_to_compare>;
        debugger;
        return !array.includes(this.value);

      case 'HAS_DAY':
        value_date = this.value as Date;
        return value_date.getUTCDate() === this.value_to_compare;

      case 'HAS_MONTH':
        value_date = this.value as Date;
        return value_date.getUTCMonth() + 1 === this.value_to_compare;

      case 'HAS_YEAR':
        value_date = this.value as Date;
        return value_date.getUTCFullYear() === this.value_to_compare;

      case 'IN_WEEK':
        value_date = this.value as Date;
        value_to_compare_date = new Date(this.value_to_compare as number);

        return isSameWeek(value_to_compare_date, value_date);

      default:
        return this.value === this.value_to_compare;
    }
  }
}
